@@ -1,4 +1,4 @@
-require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
+require File.expand_path('../gemutilities', __FILE__)
 require 'rubygems'
 require 'rubygems/gem_openssl'
 require 'rubygems/installer'
@@ -16,6 +16,8 @@ class TestGem < RubyGemTestCase
                       else
                         %r|/[Rr]uby/[Gg]ems/[0-9.]+|
                       end
+
+    util_remove_interrupt_command
   end
 
   def test_self_all_load_paths
@@ -95,7 +97,7 @@ class TestGem < RubyGemTestCase
     bindir = if defined?(RUBY_FRAMEWORK_VERSION) then
                '/usr/bin'
              else
-               Config::CONFIG['bindir']
+               RbConfig::CONFIG['bindir']
              end
 
     assert_equal bindir, Gem.bindir(default)
@@ -181,16 +183,11 @@ class TestGem < RubyGemTestCase
   end
 
   def test_self_default_sources
-    assert_equal %w[http://gems.rubyforge.org/], Gem.default_sources
+    assert_equal %w[http://rubygems.org/], Gem.default_sources
   end
 
   def test_self_dir
     assert_equal @gemhome, Gem.dir
-
-    Gem::DIRECTORIES.each do |filename|
-      assert File.directory?(File.join(Gem.dir, filename)),
-             "expected #{filename} to exist"
-    end
   end
 
   def test_self_ensure_gem_directories
@@ -263,6 +260,8 @@ class TestGem < RubyGemTestCase
 
   def test_self_find_files
     discover_path = File.join 'lib', 'foo', 'discover.rb'
+    cwd = File.expand_path '..', __FILE__
+    $LOAD_PATH.unshift cwd.dup
 
     foo1 = quick_gem 'foo', '1' do |s|
       s.files << discover_path
@@ -286,12 +285,14 @@ class TestGem < RubyGemTestCase
     Gem.searcher = nil
 
     expected = [
-      File.expand_path('foo/discover.rb', File.dirname(__FILE__)),
+      File.expand_path('../foo/discover.rb', __FILE__),
       File.join(foo2.full_gem_path, discover_path),
       File.join(foo1.full_gem_path, discover_path),
     ]
 
     assert_equal expected, Gem.find_files('foo/discover')
+  ensure
+    assert_equal cwd, $LOAD_PATH.shift
   end
 
   def test_self_latest_load_paths
@@ -442,13 +443,13 @@ class TestGem < RubyGemTestCase
   def test_self_refresh
     util_make_gems
 
-    a1_spec = File.join @gemhome, "specifications", "#{@a1.full_name}.gemspec" 
+    a1_spec = File.join @gemhome, "specifications", @a1.spec_name
 
     FileUtils.mv a1_spec, @tempdir
 
     refute Gem.source_index.gems.include?(@a1.full_name)
 
-    FileUtils.mv File.join(@tempdir, "#{@a1.full_name}.gemspec"), a1_spec
+    FileUtils.mv File.join(@tempdir, @a1.spec_name), a1_spec
 
     Gem.refresh
 
@@ -538,11 +539,11 @@ class TestGem < RubyGemTestCase
     path = [@userhome, other].join File::PATH_SEPARATOR
     Gem.send :set_paths, path
 
-    assert File.exist?(File.join(@userhome, 'gems'))
-    assert File.exist?(File.join(other, 'gems'))
+    assert_equal [@userhome, other, @gemhome], Gem.path
   end
 
   def test_self_set_paths_nonexistent_home
+    ENV['GEM_HOME'] = @gemhome
     Gem.clear_paths
 
     other = File.join @tempdir, 'other'
@@ -551,7 +552,7 @@ class TestGem < RubyGemTestCase
 
     Gem.send :set_paths, other
 
-    refute File.exist?(File.join(other, 'gems'))
+    assert_equal [other, @gemhome], Gem.path
   end
 
   def test_self_source_index
@@ -616,6 +617,38 @@ class TestGem < RubyGemTestCase
     ENV['USERPROFILE'] = orig_user_profile
     ENV['USERDRIVE'] = orig_user_drive
     ENV['USERPATH'] = orig_user_path
+  end if '1.9' > RUBY_VERSION
+
+  def test_load_plugins
+    with_plugin('load') { Gem.load_plugins }
+    assert_equal :loaded, TEST_PLUGIN_LOAD
+
+    util_remove_interrupt_command
+
+    # Should attempt to cause a StandardError
+    with_plugin('standarderror') { Gem.load_plugins }
+    assert_equal :loaded, TEST_PLUGIN_STANDARDERROR
+
+    util_remove_interrupt_command
+
+    # Should attempt to cause an Exception
+    with_plugin('exception') { Gem.load_plugins }
+    assert_equal :loaded, TEST_PLUGIN_EXCEPTION
+  end
+
+  def with_plugin(path)
+    test_plugin_path = File.expand_path "../plugin/#{path}", __FILE__
+
+    # A single test plugin should get loaded once only, in order to preserve
+    # sane test semantics.
+    refute_includes $LOAD_PATH, test_plugin_path
+    $LOAD_PATH.unshift test_plugin_path
+
+    capture_io do
+      yield
+    end
+  ensure
+    $LOAD_PATH.delete test_plugin_path
   end
 
   def util_ensure_gem_dirs
@@ -666,5 +699,10 @@ class TestGem < RubyGemTestCase
       defined?(@RUBY_REVISION)
   end
 
+  def util_remove_interrupt_command
+    Gem::Commands.send :remove_const, :InterruptCommand if
+      Gem::Commands.const_defined? :InterruptCommand
+  end
+
 end
 
