@@ -1,10 +1,4 @@
-#--
-# Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
-# All rights reserved.
-# See LICENSE.txt for permissions.
-#++
-
-require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
+require File.expand_path('../gemutilities', __FILE__)
 require 'rubygems/command_manager'
 
 class TestGemCommandManager < RubyGemTestCase
@@ -16,6 +10,8 @@ class TestGemCommandManager < RubyGemTestCase
   end
 
   def test_run_interrupt
+    Gem.load_plugins
+
     use_ui @ui do
       assert_raises MockGemUi::TermError do
         @command_manager.run 'interrupt'
@@ -25,6 +21,18 @@ class TestGemCommandManager < RubyGemTestCase
     end
   end
 
+  def test_run_crash_command
+    @command_manager.register_command :crash
+    use_ui @ui do
+      assert_raises MockGemUi::TermError do
+        @command_manager.run 'crash'
+      end
+      assert_equal '', ui.output
+      err = ui.error.split("\n").first
+      assert_equal "ERROR:  Loading command: crash (RuntimeError)", err
+    end
+  end
+
   def test_process_args_bad_arg
     use_ui @ui do
       assert_raises(MockGemUi::TermError) {
