@@ -1,4 +1,4 @@
-require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
+require File.expand_path('../gemutilities', __FILE__)
 require 'rubygems/commands/contents_command'
 
 class TestGemCommandsContentsCommand < RubyGemTestCase
@@ -139,13 +139,15 @@ Rakefile
   end
 
   def test_handle_options
-    assert_equal false, @cmd.options[:lib_only]
-    assert_equal [], @cmd.options[:specdirs]
-    assert_equal nil, @cmd.options[:version]
+    refute @cmd.options[:lib_only]
+    assert @cmd.options[:prefix]
+    assert_empty @cmd.options[:specdirs]
+    assert_nil @cmd.options[:version]
 
-    @cmd.send :handle_options, %w[-l -s foo --version 0.0.2]
+    @cmd.send :handle_options, %w[-l -s foo --version 0.0.2 --no-prefix]
 
-    assert_equal true, @cmd.options[:lib_only]
+    assert @cmd.options[:lib_only]
+    refute @cmd.options[:prefix]
     assert_equal %w[foo], @cmd.options[:specdirs]
     assert_equal Gem::Requirement.new('0.0.2'), @cmd.options[:version]
   end
