@@ -1,12 +1,4 @@
-#!/usr/bin/env ruby
-#--
-# Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
-# All rights reserved.
-# See LICENSE.txt for permissions.
-#++
-
-require 'test/unit'
-require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
+require File.expand_path('../gemutilities', __FILE__)
 require 'rubygems/package'
 
 class TestKernel < RubyGemTestCase
@@ -33,14 +25,14 @@ class TestKernel < RubyGemTestCase
 
   def test_gem_redundent
     assert gem('a', '= 1'), "Should load"
-    assert ! gem('a', '= 1'), "Should not load"
+    refute gem('a', '= 1'), "Should not load"
     assert_equal 1, $:.select { |p| %r{a-1/lib} =~ p }.size
     assert_equal 1, $:.select { |p| %r{a-1/bin} =~ p }.size
   end
 
   def test_gem_overlapping
     assert gem('a', '= 1'), "Should load"
-    assert ! gem('a', '>= 1'), "Should not load"
+    refute gem('a', '>= 1'), "Should not load"
     assert_equal 1, $:.select { |p| %r{a-1/lib} =~ p }.size
     assert_equal 1, $:.select { |p| %r{a-1/bin} =~ p }.size
   end
@@ -48,17 +40,19 @@ class TestKernel < RubyGemTestCase
   def test_gem_conflicting
     assert gem('a', '= 1'), "Should load"
 
-    ex = assert_raise Gem::Exception do
+    ex = assert_raises Gem::LoadError do
       gem 'a', '= 2'
     end
 
-    assert_match(/activate a \(= 2\)/, ex.message)
+    assert_match(/activate a \(= 2, runtime\)/, ex.message)
     assert_match(/activated a-1/, ex.message)
+    assert_equal 'a', ex.name
+    assert_equal Gem::Requirement.new('= 2'), ex.version_requirement
 
     assert $:.any? { |p| %r{a-1/lib} =~ p }
     assert $:.any? { |p| %r{a-1/bin} =~ p }
-    assert ! $:.any? { |p| %r{a-2/lib} =~ p }
-    assert ! $:.any? { |p| %r{a-2/bin} =~ p }
+    refute $:.any? { |p| %r{a-2/lib} =~ p }
+    refute $:.any? { |p| %r{a-2/bin} =~ p }
   end
 
 end
